searchData={"items":[{"type":"module","doc":"Peri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema.\nIt focuses on validating raw maps and supports nested schemas and optional fields.","title":"Peri","ref":"Peri.html"},{"type":"module","doc":"To define a schema, use the `defschema` macro. By default, all fields in the schema are optional unless specified as `{:required, type}`.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string},\n    address: %{\n      street: :string,\n      city: :string\n    },\n    tags: {:list, :string},\n    role: {:enum, [:admin, :user, :guest]},\n    geolocation: {:tuple, [:float, :float]},\n    rating: {:custom, &validate_rating/1}\n  }\n\n  defp validate_rating(n) when n < 10, do: :ok\n  defp validate_rating(_), do: {:error, \"invalid rating\", []}\nend\n```\n\nYou can then use the schema to validate data:\n\n```elixir\nuser_data = %{name: \"John\", age: 30, email: \"john@example.com\", address: %{street: \"123 Main St\", city: \"Somewhere\"}, tags: [\"science\", \"funky\"], role: :admin, geolocation: {12.2, 34.2}, rating: 9}\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","title":"Usage - Peri","ref":"Peri.html#module-usage"},{"type":"module","doc":"- `:string` - Validates that the field is a binary (string).\n- `:integer` - Validates that the field is an integer.\n- `:float` - Validates that the field is a float.\n- `:boolean` - Validates that the field is a boolean.\n- `:atom` - Validates that the field is an atom.\n- `:any` - Allow any datatype.\n- `{:required, type}` - Marks the field as required and validates it according to the specified type.\n- `:map` - Validates that the field is a map without checking nested schema.\n- `{:either, {type_1, type_2}}` - Validates that the field is either of `type_1` or `type_2`.\n- `{:oneof, types}` - Validates that the field is at least one of the provided types.\n- `{:list, type}` - Validates that the field is a list where elements belongs to a determined type.\n- `{:tuple, types}` - Validates that the field is a tuple with determined size and each element have your own type validation (sequential).\n- `{custom, anonymous_fun_arity_1}` - Validates that the field passes on the callback, the function needs to return either `:ok` or `{:error, reason}` where `reason` should be a string.\n- `{:custom, {MyModule, :my_validation}}` - Same as `{custom, anonymous_fun_arity_1}` but you pass a remote module and a function name as atom.\n- `{:custom, {MyModule, :my_validation, [arg1, arg2]}}` - Same as `{:custom, {MyModule, :my_validation}}` but you can pass extra arguments to your validation function. Note that the value of the field is always the first argument.","title":"Available Types - Peri","ref":"Peri.html#module-available-types"},{"type":"macro","doc":"Defines a schema with a given name and schema definition.","title":"Peri.defschema/2","ref":"Peri.html#defschema/2"},{"type":"macro","doc":"defmodule MySchemas do\n      import Peri\n\n      defschema :user, %{\n        name: :string,\n        age: :integer,\n        email: {:required, :string}\n      }\n    end\n\n    user_data = %{name: \"John\", age: 30, email: \"john@example.com\"}\n    MySchemas.user(user_data)\n    # => {:ok, %{name: \"John\", age: 30, email: \"john@example.com\"}}\n\n    invalid_data = %{name: \"John\", age: 30}\n    MySchemas.user(invalid_data)\n    # => {:error, [email: \"is required\"]}","title":"Examples - Peri.defschema/2","ref":"Peri.html#defschema/2-examples"},{"type":"function","doc":"Validates a given data map against a schema.\n\nReturns `{:ok, data}` if the data is valid according to the schema, or `{:error, errors}` if there are validation errors.","title":"Peri.validate/2","ref":"Peri.html#validate/2"},{"type":"function","doc":"- schema: The schema definition map.\n  - data: The data map to be validated.","title":"Parameters - Peri.validate/2","ref":"Peri.html#validate/2-parameters"},{"type":"function","doc":"schema = %{\n      name: :string,\n      age: :integer,\n      email: {:required, :string}\n    }\n\n    data = %{name: \"John\", age: 30, email: \"john@example.com\"}\n    Peri.validate(schema, data)\n    # => {:ok, %{name: \"John\", age: 30, email: \"john@example.com\"}}\n\n    invalid_data = %{name: \"John\", age: 30}\n    Peri.validate(schema, invalid_data)\n    # => {:error, [email: \"is required\"]}","title":"Examples - Peri.validate/2","ref":"Peri.html#validate/2-examples"},{"type":"module","doc":"","title":"Peri.Error","ref":"Peri.Error.html"},{"type":"extras","doc":"# Peri\n\nPeri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema. It focuses on validating raw maps and supports nested schemas and optional fields. With Peri, you can define schemas and validate data structures in a concise and readable manner.","title":"Peri","ref":"readme.html"},{"type":"extras","doc":"- Define schemas using a simple and intuitive syntax.\n- Validate data structures against schemas.\n- Support for nested schemas.\n- Optional and required fields.","title":"Features - Peri","ref":"readme.html#features"},{"type":"extras","doc":"To use Peri in your project, add it to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:peri, \"~> 0.2\"}\n  ]\nend\n```\n\nThen, run `mix deps.get` to fetch the dependencies.","title":"Installation - Peri","ref":"readme.html#installation"},{"type":"extras","doc":"","title":"Usage - Peri","ref":"readme.html#usage"},{"type":"extras","doc":"To define a schema, use the `defschema` macro. By default, all fields in the schema are optional unless specified as `{:required, type}`.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :product, %{\n    id: {:required, :integer},\n    name: {:required, :string},\n    price: :float,\n    in_stock: :boolean\n  }\nend\n```","title":"Defining Schemas - Peri","ref":"readme.html#defining-schemas"},{"type":"extras","doc":"You can then use the schema to validate data:\n\n```elixir\nproduct_data = %{id: 1, name: \"Laptop\", price: 999.99, in_stock: true}\ncase MySchemas.product(product_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","title":"Validating Data - Peri","ref":"readme.html#validating-data"},{"type":"extras","doc":"Peri supports the following types for schema definitions:\n\n  - `:string` - Validates that the field is a binary (string).\n  - `:integer` - Validates that the field is an integer.\n  - `:float` - Validates that the field is a float.\n  - `:boolean` - Validates that the field is a boolean.\n  - `:atom` - Validates that the field is an atom.\n  - `:any` - Allow any datatype.\n  - `{:required, type}` - Marks the field as required and validates it according to the specified type.\n  - `:map` - Validates that the field is a map without checking nested schema.\n  - `{:either, {type_1, type_2}}` - Validates that the field is either of `type_1` or `type_2`.\n  - `{:oneof, types}` - Validates that the field is at least one of the provided types.\n  - `{:list, type}` - Validates that the field is a list where elements belongs to a determined type.\n  - `{:tuple, types}` - Validates that the field is a tuple with determined size and each element have your own type validation (sequential).\n  - `{custom, anonymous_fun_arity_1}` - Validates that the field passes on the callback, the function needs to return either `:ok` or `{:error, template, info}` where `template` should be a string with optinal EEx directives and `info` is a list of EEx mapping (or an empty list). This allow supporting composable schemas. See on the [Composable and Recursive Schemas](#composable-and-recursive-schemas) section.\n  - `{:custom, {MyModule, :my_validation}}` - Same as `{custom, anonymous_fun_arity_1}` but you pass a remote module and a function name as atom.\n  - `{:custom, {MyModule, :my_validation, [arg1, arg2]}}` - Same as `{:custom, {MyModule, :my_validation}}` but you can pass extra arguments to your validation function. Note that the value of the field is always the first argument.\n  - Nested maps with schema defined","title":"Available Types - Peri","ref":"readme.html#available-types"},{"type":"extras","doc":"","title":"Examples - Peri","ref":"readme.html#examples"},{"type":"extras","doc":"```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :tag, :string\nend\n\ndata = \"valid!\"\ncase MySchemas.order(data) do\n  {:ok, ^data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n\ninvalid = 42\ncase MySchemas.order(data) do\n  {:ok, valid} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","title":"Simple Schema Validation - Peri","ref":"readme.html#simple-schema-validation"},{"type":"extras","doc":"```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :order, %{\n    order_id: {:required, :integer},\n    customer_name: {:required, :string},\n    total_amount: :float\n  }\nend\n\norder_data = %{order_id: 123, customer_name: \"Alice\", total_amount: 59.99}\ncase MySchemas.order(order_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n\ninvalid_order_data = %{order_id: 123}\ncase MySchemas.order(invalid_order_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","title":"Simple Map Schema Validation - Peri","ref":"readme.html#simple-map-schema-validation"},{"type":"extras","doc":"You don't even need to use the `defschema/2` macro, jsut define the schema as you want and pass them with data to `Peri/validate/2` (actually is exactly that the macro does: creates a local function with the name of the schema and pass along the schema, receiving data as extra argument.)\n\n```elixir\ndefmodule MySchemas do\n  @raw_user_schema %{age: :integer, name: :string}\n\n  def create_user(data) do\n    with {:ok, data} <- Peri.validate(@raw_user_schema, data) do\n      # rest of function ....\n    end\n  end\nend\n```","title":"Raw Schema Validation - Peri","ref":"readme.html#raw-schema-validation"},{"type":"extras","doc":"```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user_profile, %{\n    username: {:required, :string},\n    email: {:required, :string},\n    details: %{\n      age: :integer,\n      bio: :string\n    }\n  }\nend\n\nprofile_data = %{\n  username: \"bob_smith\",\n  email: \"bob@example.com\",\n  details: %{age: 30, bio: \"Software developer\"}\n}\n\ncase MySchemas.user_profile(profile_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n\ninvalid_profile_data = %{\n  username: \"bob_smith\",\n  email: \"bob@example.com\",\n  details: %{age: \"thirty\"}\n}\n\ncase MySchemas.user_profile(invalid_profile_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","title":"Nested Schema Validation - Peri","ref":"readme.html#nested-schema-validation"},{"type":"extras","doc":"#### Custom Schemas validations\n\nYou can provide another schemas or custom function validations as nested validations. The idea is to provide a function that receives a value and returns `:ok` in case of success and `{:error, template, info}` in case of error.\n\nLet's take a look into an example of this definition:\n\n```elixir\ndefmodule CustomValidations do\n  @spec positive?(term) :: :ok | {:error, String.t(), keyword}\n  def positive?(val) when is_integer(val) and val > 0, do: :ok\n  def positive?(_val), do: {:error, \"must be positive\", []}\n\n  @spec starts_with_a?(term) :: :ok | {:error, String.t(), keyword}\n  def starts_with_a?(<<\"a\", _::binary>>), do: :ok\n  def starts_with_a?(_val), do: {:error, \"must start with <%= prefix %>\", [prefix: \"'a'\"]}\nend\n\ndefmodule MySchemas do\n  import Peri\n\n  defschema :custom_example, %{\n    positive_number: {:custom, &CustomValidations.positive?/1},\n    name: {:custom, {CustomValidations, :starts_with_a?}}\n  }\nend\n\ndata = %{positive_number: 5, name: \"alice\"}\ncase MySchemas.custom_example(data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n\ninvalid_data = %{positive_number: -5, name: \"bob\"}\ncase MySchemas.custom_example(invalid_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```\n\nThe `template` is a raw EEx string with directives that references any variable on the `info` payload, which is a keyword list that represents variables. You can know more about error handling on the [Error Handling](#error-handling) section.\n\n#### Composable Schemas\n\nComposable schemas allow you to define reusable schema components and combine them to create more complex schemas. As Peri's schemas are just functions (and under the hoods simple elixir data), you can pass schemas around to another schemas.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  # Define reusable schemas\n  defschema :string_list, {:list, :string}\n  defschema :user_info, %{username: :string, email: {:required, {:custom, &string_list/1}}}\n  defschema :user_data, %{user_id: :integer, info: {:custom, &user_info/1}}\n\n  # Example usage\n  def example_usage do\n    data = [\"hello\", \"world\"]\n    case string_list(data) do\n      {:ok, valid_data} -> IO.puts(\"StringList is valid!\")\n      {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n    end\n\n    user_data_example = %{user_id: 1, info: %{username: \"john_doe\", email: \"john@example.com\"}}\n    case user_data(user_data_example) do\n      {:ok, valid_data} -> IO.puts(\"UserData is valid!\")\n      {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n    end\n  end\nend\n```\n\nAnother way is to define those composable schemas as raw data maps (or even other data types)\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  # Define reusable schemas\n  @string_list {:list, :string}\n  @user_info %{username: :string, email: {:required, @string_list}}\n  defschema :user_data, %{user_id: :integer, info: @user_info}\n\n  # Example usage\n  def example_usage do\n    data = [\"hello\", \"world\"]\n    case string_list(data) do\n      {:ok, valid_data} -> IO.puts(\"StringList is valid!\")\n      {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n    end\n\n    user_data_example = %{user_id: 1, info: %{username: \"john_doe\", email: \"john@example.com\"}}\n    case user_data(user_data_example) do\n      {:ok, valid_data} -> IO.puts(\"UserData is valid!\")\n      {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n    end\n  end\nend\n```\n\n#### Recursive Schemas\n\nRecursive schemas allow you to define schemas that reference themselves, enabling the validation of nested and hierarchical data structures.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  # Define a recursive schema\n  defschema :category, %{\n    id: :integer,\n    name: :string,\n    subcategories: {:list, {:custom, &category/1}}\n  }\n\n  # Example usage\n  def example_usage do\n    category_data = %{\n      id: 1,\n      name: \"Electronics\",\n      subcategories: [\n        %{id: 2, name: \"Computers\", subcategories: []},\n        %{id: 3, name: \"Phones\", subcategories: [%{id: 4, name: \"Smartphones\", subcategories: []}]}\n      ]\n    }\n\n    case category(category_data) do\n      {:ok, valid_data} -> IO.puts(\"Category is valid!\")\n      {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n    end\n  end\nend\n```","title":"Composable and Recursive Schemas - Peri","ref":"readme.html#composable-and-recursive-schemas"},{"type":"extras","doc":"For each error in you schema, it will be constructed a `%Peri.Error{}` struct, that is defined as:\n\n```elixir\n@type t :: %Peri.Error{\n  path: list(atom), # the \"path\" to the error key inside your data\n  key: atom, # the error key in your data\n  message: String.t(), # already evaluated EEx string defining the error message\n  content: keyword, # the data that filled the error message, variables and additonal data\n  errors: list(Peri.Error.t()) | nil # if your structure is nested, it can have nested errors on your data\n}\n```","title":"Error Handling - Peri","ref":"readme.html#error-handling"},{"type":"extras","doc":"While both Peri and Ecto provide mechanisms for working with schemas in Elixir, they serve different purposes and are used in different contexts.","title":"Comparison: Peri vs. Ecto - Peri","ref":"readme.html#comparison-peri-vs-ecto"},{"type":"extras","doc":"- **Purpose**: Peri is designed specifically for schema validation. It focuses on validating raw maps against defined schemas.\n- **Flexibility**: Peri allows for easy validation of nested structures and optional fields.\n- **Simplicity**: The syntax for defining schemas in Peri is simple and intuitive, making it easy to use for straightforward validation tasks.\n- **Use Case**: Ideal for validating data structures in contexts where you don't need the full power of a database ORM.","title":"Peri - Peri","ref":"readme.html#peri"},{"type":"extras","doc":"- **Purpose**: Ecto is a comprehensive database wrapper and query generator for Elixir. It provides tools for defining schemas, querying databases, and managing changesets.\n- **Complexity**: Ecto is more complex due to its broader feature set, which includes support for migrations, associations, and transactions.\n- **Schema Definitions**: Ecto schemas are typically tied to database tables, with a focus on struct-based data manipulation.\n- **Use Case**: Ideal for applications that require robust interaction with a database, including data validation, querying, and persistence.\n\n#### Summary\n- Use **Peri** if you need a lightweight, flexible library for validating raw maps and nested data structures without the overhead of database interactions.\n- Use **Ecto** if you need a powerful tool for managing database schemas, querying, and data persistence, and if you require comprehensive features like associations, migrations, and transactions.\n\nBy understanding the different strengths and use cases of Peri and Ecto, you can choose the right tool for your specific needs, ensuring efficient and effective data handling in your Elixir applications.","title":"Ecto - Peri","ref":"readme.html#ecto"},{"type":"extras","doc":"The name \"Peri\" is derived from the prefix \"peri-\" which means \"around\" or \"surrounding\". This reflects the library's purpose of providing a protective layer around your data structures, ensuring they conform to specified schemas. Just as a perimeter protects and defines boundaries, Peri ensures that your data is validated and well-defined according to your schemas.","title":"Why \"Peri\"? - Peri","ref":"readme.html#why-peri"},{"type":"extras","doc":"We welcome contributions to Peri! If you have suggestions for improvements or find bugs, please open an issue or submit a pull request on GitHub.","title":"Contributing - Peri","ref":"readme.html#contributing"},{"type":"extras","doc":"Peri is released under the MIT License. See the LICENSE file for more details.\n\n---\n\nPeri makes it easy to define and validate data structures in Elixir, providing a powerful tool for ensuring data integrity in your applications. Start using Peri today and enjoy simple, reliable schema validation!","title":"License - Peri","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}